# frozen_string_literal: true
module Hermes
  module Api
    module Dmo
      class RexController < ::Api::D2::ApplicationController
        def for_lessor?
          for_lessor = params[:for_lessor]
          for_lessor.present? && ['true', true].include?(for_lessor)
        end

        def target_class_name
          for_lessor? ? 'lessor' : 'entity'
        end

        def from_parent_depth_param
          Integer(params[:parent_depth]) rescue 0
        end

        def max_depth_param
          Integer(params[:max_depth]) rescue -1
        end

        def class_target(for_lessor = nil)
          for_lessor ? Hermes::Lessor : Hermes::Entity
        end

        def from_id
          @from_id ||= params[:id]
        end

        def for_lessor_unauthorized_auth_roles
          for_lessor? && !(
            current_user.auth_roles&.include?('superadministrateur') || 
            current_user.auth_roles&.include?('bailleur') || 
            current_user.auth_roles&.include?('rdsl') || 
            current_user.auth_roles&.include?('cre')
          )
        end

        def root_elements
          return render_json_error(:unprocessable_entity, :not_lessor) if for_lessor_unauthorized_auth_roles


          possible_root_ids  = class_target(for_lessor?).readable_by(current_user)
                                                        .pluck(:root_id)
                                                        .uniq
                                                        .compact ||
                                                        []
          root_ids = possible_root_ids.map do |r|
            entity = class_target(for_lessor?).readable_by(current_user).find_by(root_id: r)
            maximum_root_for_user(entity)
          end

          render(
            jsonapi: class_target(for_lessor?).where(:id.in => root_ids),
            each_serializer: Hermes::Rex::RootElementSerializer
          )
        end

        def maximum_root_for_user(entity)
          return entity.id if entity.is_root || !class_target(for_lessor?).readable_by(current_user).find_by(id: entity.parent_id)

          maximum_root_for_user(entity.parent)
        end

        def show
          return render_json_error(:unprocessable_entity, :missing_id_parameter) unless from_id.present?
          return render_json_error(:unprocessable_entity, :not_lessor) if for_lessor_unauthorized_auth_roles

          target = class_target(for_lessor?).find_by(id: from_id)
          return render_json_error(:not_found, :object_not_found) if target.blank?

          tree = target.tree(
                                     from_parent_depth_param,
                                     max_depth_param,
                                     target_class_name,
                                     current_user,
                                     true
                                   )

          render(
            jsonapi: {
              data: Hermes::Rex::TreeSerializer.new(
                tree,
                root: true,
                scope: current_user,
                for_lessor: for_lessor?
              )
            }
          )
        end

        def subsidiaries
          return render_json_error(:unprocessable_entity, :missing_parent_ids_parameter) unless params[:parent_ids].present? 
          return render_json_error(:unprocessable_entity, :not_lessor) if for_lessor_unauthorized_auth_roles


          
          query = {
            '$or': [
              { id: { '$in': params[:parent_ids]}},
              { parent_id: { '$in': params[:parent_ids]}},
            ]
          }

          render(
            jsonapi: class_target(for_lessor?).readable_by(current_user).and(query),
            each_serializer: for_lessor? ? Hermes::LessorSerializer : Hermes::EntitySerializer  ,
          )
        end

        def current_user_roles
          current_user.attributes.slice(:roles, :auth_roles).values.flatten.uniq.compact
        end

        def current_user
          @user
        end
      end
    end
  end
end
